@@ -2,20 +2,22 @@ use super::{AsyncPushSender, HandleContainer, RedisFuture};
 #[cfg(feature = "cache-aio")]
 use crate::caching::CacheManager;
 use crate::{
-    aio::{check_resp3, ConnectionLike, MultiplexedConnection, Runtime},
-    cmd,
+    aio::{ConnectionLike, MultiplexedConnection, Runtime},
+    check_resp3, cmd,
     subscription_tracker::{SubscriptionAction, SubscriptionTracker},
     types::{RedisError, RedisResult, Value},
-    AsyncConnectionConfig, Client, Cmd, Pipeline, PushInfo, PushKind, ToRedisArgs,
+    AsyncConnectionConfig, Client, Cmd, Pipeline, ProtocolVersion, PushInfo, PushKind, ToRedisArgs,
 };
 use arc_swap::ArcSwap;
 use backon::{ExponentialBuilder, Retryable};
 use futures_channel::oneshot;
 use futures_util::future::{self, BoxFuture, FutureExt, Shared};
-use std::sync::Arc;
+use std::sync::{Arc, Weak};
 use tokio::sync::mpsc::{unbounded_channel, UnboundedReceiver};
 use tokio::sync::Mutex;
 
+type OptionalPushSender = Option<Arc<dyn AsyncPushSender>>;
+
 /// The configuration for reconnect mechanism and request timing for the [ConnectionManager]
 #[derive(Clone)]
 pub struct ConnectionManagerConfig {
@@ -144,9 +146,6 @@ impl ConnectionManagerConfig {
     ///
     /// The sender can be a channel, or an arbitrary function that handles [crate::PushInfo] values.
     /// This will fail client creation if the connection isn't configured for RESP3 communications via the [crate::RedisConnectionInfo::protocol] field.
-    /// Setting this will mean that the connection manager actively listens to updates from the
-    /// server, and so it will cause the manager to reconnect after a disconnection, even if the manager was unused at
-    /// the time of the disconnect.
     ///
     /// # Examples
     ///
@@ -256,6 +255,9 @@ struct Internals {
 ///   initiated, will have to await the connection future.
 /// - If reconnecting fails, all pending commands will be failed as well. A
 ///   new reconnection attempt will be triggered if the error is an I/O error.
+/// - If the connection manager uses RESP3 connection,it actively listens to updates from the
+///   server, and so it will cause the manager to reconnect after a disconnection, even if the manager was unused at
+///   the time of the disconnect.
 ///
 /// [multiplexed-connection]: struct.MultiplexedConnection.html
 #[derive(Clone)]
@@ -272,7 +274,7 @@ macro_rules! reconnect_if_dropped {
     ($self:expr, $result:expr, $current:expr) => {
         if let Err(ref e) = $result {
             if e.is_unrecoverable_error() {
-                $self.reconnect($current);
+                Self::reconnect(Arc::downgrade(&$self.0), $current);
             }
         }
     };
@@ -284,7 +286,7 @@ macro_rules! reconnect_if_io_error {
     ($self:expr, $result:expr, $current:expr) => {
         if let Err(e) = $result {
             if e.is_io_error() {
-                $self.reconnect($current);
+                Self::reconnect(Arc::downgrade(&$self.0), $current);
             }
             return Err(e);
         }
@@ -418,7 +420,13 @@ impl ConnectionManager {
             );
 
             let (internal_sender, internal_receiver) = unbounded_channel();
-            components_for_reconnection_on_push = Some((internal_receiver, push_sender));
+            components_for_reconnection_on_push = Some((internal_receiver, Some(push_sender)));
+
+            connection_config =
+                connection_config.set_push_sender_internal(Arc::new(internal_sender));
+        } else if client.connection_info.redis.protocol != ProtocolVersion::RESP2 {
+            let (internal_sender, internal_receiver) = unbounded_channel();
+            components_for_reconnection_on_push = Some((internal_receiver, None));
 
             connection_config =
                 connection_config.set_push_sender_internal(Arc::new(internal_sender));
@@ -446,14 +454,18 @@ impl ConnectionManager {
 
         if let Some((internal_receiver, external_sender)) = components_for_reconnection_on_push {
             oneshot_sender
-                .send((new_self.clone(), internal_receiver, external_sender))
+                .send((
+                    Arc::downgrade(&new_self.0),
+                    internal_receiver,
+                    external_sender,
+                ))
                 .map_err(|_| {
                     crate::RedisError::from((
                         crate::ErrorKind::ClientError,
                         "Failed to set automatic resubscription",
                     ))
                 })?;
-        }
+        };
 
         Ok(new_self)
     }
@@ -485,14 +497,25 @@ impl ConnectionManager {
     ///
     /// The `current` guard points to the shared future that was active
     /// when the connection loss was detected.
-    fn reconnect(&self, current: arc_swap::Guard<Arc<SharedRedisFuture<MultiplexedConnection>>>) {
+    fn reconnect(
+        internals: Weak<Internals>,
+        current: arc_swap::Guard<Arc<SharedRedisFuture<MultiplexedConnection>>>,
+    ) {
+        let Some(internals) = internals.upgrade() else {
+            return;
+        };
+        let internals_clone = internals.clone();
+        #[cfg(not(feature = "cache-aio"))]
+        let connection_config = internals.connection_config.clone();
         #[cfg(feature = "cache-aio")]
-        if let Some(manager) = self.0.cache_manager.as_ref() {
-            manager.invalidate_all();
+        let mut connection_config = internals.connection_config.clone();
+        #[cfg(feature = "cache-aio")]
+        if let Some(manager) = internals.cache_manager.as_ref() {
+            let new_cache_manager = manager.clone_and_increase_epoch();
+            connection_config = connection_config.set_cache_manager(new_cache_manager);
         }
-        let self_clone = self.clone();
         let new_connection: SharedRedisFuture<MultiplexedConnection> = async move {
-            let additional_commands = match &self_clone.0.subscription_tracker {
+            let additional_commands = match &internals_clone.subscription_tracker {
                 Some(subscription_tracker) => Some(
                     subscription_tracker
                         .lock()
@@ -501,10 +524,11 @@ impl ConnectionManager {
                 ),
                 None => None,
             };
+
             let con = Self::new_connection(
-                &self_clone.0.client,
-                self_clone.0.retry_strategy,
-                &self_clone.0.connection_config,
+                &internals_clone.client,
+                internals_clone.retry_strategy,
+                &connection_config,
                 additional_commands,
             )
             .await?;
@@ -515,23 +539,22 @@ impl ConnectionManager {
 
         // Update the connection in the connection manager
         let new_connection_arc = Arc::new(new_connection.clone());
-        let prev = self
-            .0
+        let prev = internals
             .connection
             .compare_and_swap(&current, new_connection_arc);
 
         // If the swap happened...
         if Arc::ptr_eq(&prev, &current) {
             // ...start the connection attempt immediately but do not wait on it.
-            self.0.runtime.spawn(new_connection.map(|_| ()));
+            internals.runtime.spawn(new_connection.map(|_| ())).detach();
         }
     }
 
     async fn check_for_disconnect_pushes(
         receiver: oneshot::Receiver<(
-            ConnectionManager,
+            Weak<Internals>,
             UnboundedReceiver<PushInfo>,
-            Arc<dyn AsyncPushSender>,
+            OptionalPushSender,
         )>,
     ) {
         let Ok((this, mut internal_receiver, external_sender)) = receiver.await else {
@@ -539,10 +562,13 @@ impl ConnectionManager {
         };
         while let Some(push_info) = internal_receiver.recv().await {
             if push_info.kind == PushKind::Disconnection {
-                this.reconnect(this.0.connection.load());
+                let Some(internals) = this.upgrade() else {
+                    return;
+                };
+                Self::reconnect(Arc::downgrade(&internals), internals.connection.load());
             }
-            if external_sender.send(push_info).is_err() {
-                return;
+            if let Some(sender) = external_sender.as_ref() {
+                let _ = sender.send(push_info);
             }
         }
     }
@@ -593,11 +619,14 @@ impl ConnectionManager {
         let Some(subscription_tracker) = &self.0.subscription_tracker else {
             return;
         };
-        let mut guard = subscription_tracker.lock().await;
-        guard.update_with_request(action, args.to_redis_args().into_iter());
+        let args = args.to_redis_args().into_iter();
+        subscription_tracker
+            .lock()
+            .await
+            .update_with_request(action, args);
     }
 
-    /// Subscribes to a new channel(s).    
+    /// Subscribes to a new channel(s).
     ///
     /// Updates from the sender will be sent on the push sender that was passed to the manager.
     /// If the manager was configured without a push sender, the connection won't be able to pass messages back to the user.
@@ -605,6 +634,18 @@ impl ConnectionManager {
     /// This method is only available when the connection is using RESP3 protocol, and will return an error otherwise.
     /// It should be noted that unless [ConnectionManagerConfig::set_automatic_resubscription] was called,
     /// the subscription will be removed on a disconnect and must be re-subscribed.
+    ///  
+    /// ```rust,no_run
+    /// # async fn func() -> redis::RedisResult<()> {
+    /// let client = redis::Client::open("redis://127.0.0.1/?protocol=resp3").unwrap();
+    /// let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+    /// let config = redis::aio::ConnectionManagerConfig::new().set_push_sender(tx);
+    /// let mut con = client.get_connection_manager_with_config(config).await?;
+    /// con.psubscribe("channel*_1").await?;
+    /// con.psubscribe(&["channel*_2", "channel*_3"]).await?;
+    /// # Ok(())
+    /// # }
+    /// ```
     pub async fn subscribe(&mut self, channel_name: impl ToRedisArgs) -> RedisResult<()> {
         check_resp3!(self.0.client.connection_info.redis.protocol);
         let mut cmd = cmd("SUBSCRIBE");
@@ -637,6 +678,18 @@ impl ConnectionManager {
     /// This method is only available when the connection is using RESP3 protocol, and will return an error otherwise.
     /// It should be noted that unless [ConnectionManagerConfig::set_automatic_resubscription] was called,
     /// the subscription will be removed on a disconnect and must be re-subscribed.
+    ///
+    /// ```rust,no_run
+    /// # async fn func() -> redis::RedisResult<()> {
+    /// let client = redis::Client::open("redis://127.0.0.1/?protocol=resp3").unwrap();
+    /// let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+    /// let config = redis::aio::ConnectionManagerConfig::new().set_push_sender(tx);
+    /// let mut con = client.get_connection_manager_with_config(config).await?;
+    /// con.psubscribe("channel*_1").await?;
+    /// con.psubscribe(&["channel*_2", "channel*_3"]).await?;
+    /// # Ok(())
+    /// # }
+    /// ```
     pub async fn psubscribe(&mut self, channel_pattern: impl ToRedisArgs) -> RedisResult<()> {
         check_resp3!(self.0.client.connection_info.redis.protocol);
         let mut cmd = cmd("PSUBSCRIBE");
@@ -660,7 +713,7 @@ impl ConnectionManager {
         Ok(())
     }
 
-    /// Gets [`CacheStatistics`] for current connection if caching is enabled.
+    /// Gets [`crate::caching::CacheStatistics`] for current connection if caching is enabled.
     #[cfg(feature = "cache-aio")]
     #[cfg_attr(docsrs, doc(cfg(feature = "cache-aio")))]
     pub fn get_cache_statistics(&self) -> Option<crate::caching::CacheStatistics> {
