@@ -104,7 +104,8 @@ use std::{
 mod request;
 mod routing;
 use crate::{
-    aio::{check_resp3, ConnectionLike, HandleContainer, MultiplexedConnection, Runtime},
+    aio::{ConnectionLike, HandleContainer, MultiplexedConnection, Runtime},
+    check_resp3,
     cluster::{get_connection_info, slot_cmd},
     cluster_client::ClusterParams,
     cluster_routing::{
@@ -119,6 +120,8 @@ use crate::{
     RedisFuture, RedisResult, ToRedisArgs, Value,
 };
 
+#[cfg(feature = "cache-aio")]
+use crate::caching::{CacheManager, CacheStatistics};
 use crate::ProtocolVersion;
 use futures_sink::Sink;
 use futures_util::{
@@ -137,6 +140,8 @@ struct ClientSideState {
     _task_handle: HandleContainer,
     response_timeout: Option<Duration>,
     runtime: Runtime,
+    #[cfg(feature = "cache-aio")]
+    cache_manager: Option<CacheManager>,
 }
 
 /// This represents an async Redis Cluster connection.
@@ -159,6 +164,8 @@ where
     ) -> RedisResult<ClusterConnection<C>> {
         let protocol = cluster_params.protocol.unwrap_or_default();
         let response_timeout = cluster_params.response_timeout;
+        #[cfg(feature = "cache-aio")]
+        let cache_manager = cluster_params.cache_manager.clone();
         let runtime = Runtime::locate();
         ClusterConnInner::new(initial_nodes, cluster_params)
             .await
@@ -179,6 +186,8 @@ where
                         _task_handle,
                         response_timeout,
                         runtime,
+                        #[cfg(feature = "cache-aio")]
+                        cache_manager,
                     }),
                 }
             })
@@ -271,6 +280,19 @@ where
     /// This method is only available when the connection is using RESP3 protocol, and will return an error otherwise.
     /// It should be noted that the subscription will be automatically resubscribed after disconnections, so the user might
     /// receive additional pushes with [crate::PushKind::Subscribe], later after the subscription completed.
+    ///
+    /// ```rust,no_run
+    /// # async fn func() -> redis::RedisResult<()> {
+    /// let nodes = vec!["redis://127.0.0.1/protocol=3"];
+    /// let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+    /// let client = redis::cluster::ClusterClientBuilder::new(nodes)
+    ///   .use_protocol(redis::ProtocolVersion::RESP3)
+    ///   .push_sender(tx).build()?;
+    /// let mut con = client.get_async_connection().await?;
+    /// con.subscribe(&["channel_1", "channel_2"]).await?;
+    /// con.unsubscribe(&["channel_1", "channel_2"]).await?;
+    /// # Ok(()) }
+    /// ```
     pub async fn subscribe(&mut self, channel_name: impl ToRedisArgs) -> RedisResult<()> {
         check_resp3!(self.state.protocol);
         let mut cmd = cmd("SUBSCRIBE");
@@ -298,6 +320,21 @@ where
     /// This method is only available when the connection is using RESP3 protocol, and will return an error otherwise.
     /// It should be noted that the subscription will be automatically resubscribed after disconnections, so the user might
     /// receive additional pushes with [crate::PushKind::PSubscribe], later after the subscription completed.
+    ///
+    /// ```rust,no_run
+    /// # #[cfg(feature = "aio")]
+    /// # async fn do_something() -> redis::RedisResult<()> {
+    /// let nodes = vec!["redis://127.0.0.1/protocol=3"];
+    /// let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+    /// let client = redis::cluster::ClusterClientBuilder::new(nodes)
+    ///   .use_protocol(redis::ProtocolVersion::RESP3)
+    ///   .push_sender(tx).build()?;
+    /// let mut connection = client.get_async_connection().await?;
+    /// connection.psubscribe("channel*_1").await?;
+    /// connection.psubscribe(&["channel*_2", "channel*_3"]).await?;
+    /// # Ok(())
+    /// # }
+    /// ```
     pub async fn psubscribe(&mut self, channel_pattern: impl ToRedisArgs) -> RedisResult<()> {
         check_resp3!(self.state.protocol);
         let mut cmd = cmd("PSUBSCRIBE");
@@ -325,6 +362,19 @@ where
     /// This method is only available when the connection is using RESP3 protocol, and will return an error otherwise.
     /// It should be noted that the subscription will be automatically resubscribed after disconnections, so the user might
     /// receive additional pushes with [crate::PushKind::SSubscribe], later after the subscription completed.
+    ///
+    /// ```rust,no_run
+    /// # async fn func() -> redis::RedisResult<()> {
+    /// let nodes = vec!["redis://127.0.0.1/protocol=3"];
+    /// let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+    /// let client = redis::cluster::ClusterClientBuilder::new(nodes)
+    ///   .use_protocol(redis::ProtocolVersion::RESP3)
+    ///   .push_sender(tx).build()?;
+    /// let mut connection = client.get_async_connection().await?;
+    /// connection.ssubscribe(&["channel_1", "channel_2"]).await?;
+    /// connection.sunsubscribe(&["channel_1", "channel_2"]).await?;
+    /// # Ok(()) }
+    /// ```
     pub async fn ssubscribe(&mut self, channel_name: impl ToRedisArgs) -> RedisResult<()> {
         check_resp3!(self.state.protocol);
         let mut cmd = cmd("SSUBSCRIBE");
@@ -343,6 +393,12 @@ where
         cmd.exec_async(self).await?;
         Ok(())
     }
+    /// Gets [`CacheStatistics`] for cluster connection if caching is enabled.
+    #[cfg(feature = "cache-aio")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "cache-aio")))]
+    pub fn get_cache_statistics(&self) -> Option<CacheStatistics> {
+        self.state.cache_manager.as_ref().map(|cm| cm.statistics())
+    }
 }
 
 type ConnectionMap<C> = HashMap<String, C>;
@@ -466,7 +522,7 @@ where
                     match result {
                         Ok(conn) => Ok((addr, conn)),
                         Err(e) => {
-                            debug!("Failed to connect to initial node: {:?}", e);
+                            debug!("Failed to connect to initial node: {e:?}");
                             Err(e)
                         }
                     }
@@ -639,7 +695,7 @@ where
     fn build_slot_map(slot_map: &mut SlotMap, slots_data: Vec<Slot>) -> RedisResult<()> {
         slot_map.clear();
         slot_map.fill_slots(slots_data);
-        trace!("{:?}", slot_map);
+        trace!("{slot_map:?}");
         Ok(())
     }
 
@@ -1044,7 +1100,7 @@ where
                 };
             match request_handling {
                 Some(Retry::MoveToPending { request }) => {
-                    self.inner.pending_requests.lock().unwrap().push(request)
+                    self.inner.pending_requests.lock().unwrap().push(request);
                 }
                 Some(Retry::Immediately { request }) => {
                     let future = Self::try_request(request.cmd.clone(), self.inner.clone());
@@ -1091,8 +1147,12 @@ where
                 (*request)
                     .as_mut()
                     .respond(Err(self.refresh_error.take().unwrap()));
-            } else if let Some(request) = self.inner.pending_requests.lock().unwrap().pop() {
-                request.sender.send(Err(self.refresh_error.take().unwrap()));
+            } else {
+                // Use a separate binding for this to release the lock guard before calling send.
+                let maybe_request = self.inner.pending_requests.lock().unwrap().pop();
+                if let Some(request) = maybe_request {
+                    request.sender.send(Err(self.refresh_error.take().unwrap()));
+                }
             }
         }
     }
@@ -1319,6 +1379,8 @@ where
     let push_sender = params.async_push_sender.clone();
     let tcp_settings = params.tcp_settings.clone();
     let dns_resolver = params.async_dns_resolver.clone();
+    #[cfg(feature = "cache-aio")]
+    let cache_manager = params.cache_manager.clone();
     let info = get_connection_info(node, params)?;
     let mut config = AsyncConnectionConfig::default()
         .set_connection_timeout(connection_timeout)
@@ -1332,10 +1394,14 @@ where
     if let Some(resolver) = dns_resolver {
         config = config.set_dns_resolver_internal(resolver.clone());
     }
+    #[cfg(feature = "cache-aio")]
+    if let Some(cache_manager) = cache_manager {
+        config = config.set_cache_manager(cache_manager.clone_and_increase_epoch());
+    }
     let mut conn = match C::connect_with_config(info, config).await {
         Ok(conn) => conn,
         Err(err) => {
-            warn!("Failed to connect to node: {:?}, due to: {:?}", node, err);
+            warn!("Failed to connect to node: {node:?}, due to: {err:?}");
             return Err(err);
         }
     };
