@@ -81,7 +81,7 @@
 //! ```
 //!
 //! For async connections, connection pooling isn't necessary. The `MultiplexedConnection` is
-//! cloneable and can be used safely from multiple threads, so a single connection can be easily
+//! cheap to clone and can be used safely concurrently from multiple threads, so a single connection can be easily
 //! reused. For automatic reconnections consider using `ConnectionManager` with the `connection-manager` feature.
 //! Async cluster connections also don't require pooling and are thread-safe and reusable.
 //!
@@ -108,7 +108,7 @@
 //! * `uuid`: enables type conversion to UUID (optional)
 //! * `sentinel`: enables high-level interfaces for communication with Redis sentinels (optional)
 //! * `json`: enables high-level interfaces for communication with the JSON module (optional)
-//! * `cache-aio`: enables **experimental** client side caching for MultiplexedConnection (optional)
+//! * `cache-aio`: enables **experimental** client side caching for MultiplexedConnection, ConnectionManager and async ClusterConnection (optional)
 //! * `disable-client-setinfo`: disables the `CLIENT SETINFO` handshake during connection initialization
 //!
 //! ## Connection Parameters
@@ -214,12 +214,86 @@
 //! # }
 //! ```
 //!
+//! ## Pre-typed Commands
+//!
+//! In some cases, you may not have a desired return type for a high-level command, and would
+//! instead like to use defaults provided by the library, to avoid the clutter and development overhead
+//! of specifying types for each command.
+//!
+//! The library facilitates this by providing the `TypedCommands` and `AsyncTypedCommands`
+//! as alternatives to `Commands` and `AsyncCommands` respectively. These traits provide functions
+//! with pre-defined and opinionated return types. For example, `set` returns `()`, avoiding the need
+//! for developers to explicitly type each call as returning `()`.
+//!
+//! ```rust,no_run
+//! use redis::TypedCommands;
+//!
+//! fn fetch_an_integer() -> redis::RedisResult<isize> {
+//!     // connect to redis
+//!     let client = redis::Client::open("redis://127.0.0.1/")?;
+//!     let mut con = client.get_connection()?;
+//!     // `set` returns a `()`, so we don't need to specify the return type manually unlike in the previous example.
+//!     con.set("my_key", 42)?;
+//!     // `get_int` returns Result<Option<isize>>, as the key may not be found, or some error may occur.
+//!     Ok(con.get_int("my_key").unwrap().unwrap())
+//! }
+//! ```
+//!
 //! # RESP3 support
 //! Since Redis / Valkey version 6, a newer communication protocol called RESP3 is supported.
 //! Using this protocol allows the user both to receive a more varied `Value` results, for users
 //! who use the low-level `Value` type, and to receive out of band messages on the same connection. This allows the user to receive PubSub
 //! messages on the same connection, instead of creating a new PubSub connection (see "RESP3 async pubsub").
 //!
+
+//!
+//! ## RESP3 pubsub
+//! If you're targeting a Redis/Valkey server of version 6 or above, you can receive
+//! pubsub messages from it without creating another connection, by setting a push sender on the connection.
+//!
+//! ```rust,no_run
+//! # #[cfg(feature = "aio")]
+//! # {
+//! # use futures::prelude::*;
+//! # use redis::AsyncCommands;
+//!
+//! # async fn func() -> redis::RedisResult<()> {
+//! let client = redis::Client::open("redis://127.0.0.1/?protocol=resp3").unwrap();
+//! let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
+//! let config = redis::AsyncConnectionConfig::new().set_push_sender(tx);
+//! let mut con = client.get_multiplexed_async_connection_with_config(&config).await?;
+//! con.subscribe(&["channel_1", "channel_2"]).await?;
+//!
+//! loop {
+//!   println!("Received {:?}", rx.recv().await.unwrap());
+//! }
+//! # Ok(()) }
+//! # }
+//! ```
+//!
+//! sync example:
+//!
+//! ```rust,no_run
+//! # {
+//! # use redis::TypedCommands;
+//!
+//! # async fn func() -> redis::RedisResult<()> {
+//! let client = redis::Client::open("redis://127.0.0.1/?protocol=resp3").unwrap();
+//! let (tx, rx) = std::sync::mpsc::channel();
+//! let mut con = client.get_connection().unwrap();
+//! con.set_push_sender(tx);
+//! con.subscribe_resp3(&["channel_1", "channel_2"])?;
+//!
+//! loop {
+//!   std::thread::sleep(std::time::Duration::from_millis(10));
+//!   // the connection only reads when actively polled, so it must constantly send and receive requests.
+//!   _ = con.ping().unwrap();
+//!   println!("Received {:?}", rx.try_recv().unwrap());
+//! }
+//! # Ok(()) }
+//! # }
+//!  ```
+//!
 //! # Iteration Protocol
 //!
 //! In addition to sending a single query, iterators are also supported.  When
@@ -361,30 +435,6 @@
 //! # Ok(()) }
 //! ```
 //!
-//! ## RESP3 async pubsub
-//! If you're targeting a Redis/Valkey server of version 6 or above, you can receive
-//! pubsub messages from it without creating another connection, by setting a push sender on the connection.
-//!
-//! ```rust,no_run
-//! # #[cfg(feature = "aio")]
-//! # {
-//! # use futures::prelude::*;
-//! # use redis::AsyncCommands;
-//!
-//! # async fn func() -> redis::RedisResult<()> {
-//! let client = redis::Client::open("redis://127.0.0.1/?protocol=resp3").unwrap();
-//! let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
-//! let config = redis::AsyncConnectionConfig::new().set_push_sender(tx);
-//! let mut con = client.get_multiplexed_async_connection_with_config(&config).await?;
-//! con.subscribe(&["channel_1", "channel_2"]).await?;
-//!
-//! loop {
-//!   println!("Received {:?}", rx.recv().await.unwrap());
-//! }
-//! # Ok(()) }
-//! # }
-//! ```
-//!
 #![cfg_attr(
     feature = "script",
     doc = r##"
@@ -439,10 +489,11 @@ it will not automatically be loaded and retried. The script can be loaded using
 
 In addition to the synchronous interface that's been explained above there also exists an
 asynchronous interface based on [`futures`][] and [`tokio`][], [`smol`](https://docs.rs/smol/latest/smol/), or [`async-std`][].
+ All async connections are cheap to clone, and clones can be used concurrently from multiple threads.
 
 This interface exists under the `aio` (async io) module (which requires that the `aio` feature
 is enabled) and largely mirrors the synchronous with a few concessions to make it fit the
-constraints of `futures`.
+constraints of `futures`. 
 
 ```rust,no_run
 use futures::prelude::*;
@@ -552,8 +603,9 @@ pub use crate::client::Client;
 pub use crate::cmd::CommandCacheConfig;
 pub use crate::cmd::{cmd, pack_command, pipe, Arg, Cmd, Iter};
 pub use crate::commands::{
-    Commands, ControlFlow, Direction, FlushAllOptions, FlushDbOptions, HashFieldExpirationOptions,
-    LposOptions, PubSubCommands, ScanOptions, SetOptions,
+    Commands, ControlFlow, CopyOptions, Direction, FlushAllOptions, FlushDbOptions,
+    HashFieldExpirationOptions, LposOptions, PubSubCommands, ScanOptions, SetOptions,
+    SortedSetAddOptions, TypedCommands, UpdateCheck,
 };
 pub use crate::connection::{
     parse_redis_url, transaction, Connection, ConnectionAddr, ConnectionInfo, ConnectionLike,
@@ -591,6 +643,8 @@ pub use crate::types::{
     ExpireOption,
     Role,
     ReplicaInfo,
+    IntegerReplyOrNoOp,
+	ValueType,
 
     // error and result types
     RedisError,
@@ -603,13 +657,14 @@ pub use crate::types::{
     PushKind,
     VerbatimFormat,
     ProtocolVersion,
-    PushInfo
+    PushInfo,
 };
 
 #[cfg(feature = "aio")]
 #[cfg_attr(docsrs, doc(cfg(feature = "aio")))]
 pub use crate::{
-    cmd::AsyncIter, commands::AsyncCommands, parser::parse_redis_value_async, types::RedisFuture,
+    cmd::AsyncIter, commands::AsyncCommands, commands::AsyncTypedCommands,
+    parser::parse_redis_value_async, types::RedisFuture,
 };
 
 mod macros;
@@ -631,6 +686,13 @@ pub use crate::commands::JsonCommands;
 #[cfg_attr(docsrs, doc(cfg(all(feature = "json", feature = "aio"))))]
 pub use crate::commands::JsonAsyncCommands;
 
+#[cfg(feature = "vector-sets")]
+#[cfg_attr(docsrs, doc(cfg(feature = "vector-sets")))]
+pub use crate::commands::{
+    EmbeddingInput, VAddOptions, VEmbOptions, VSimOptions, VectorAddInput, VectorQuantization,
+    VectorSimilaritySearchInput,
+};
+
 #[cfg(feature = "geospatial")]
 #[cfg_attr(docsrs, doc(cfg(feature = "geospatial")))]
 pub mod geo;
@@ -699,6 +761,30 @@ mod parser;
 mod script;
 mod types;
 
+macro_rules! check_resp3 {
+    ($protocol: expr) => {
+        use crate::types::ProtocolVersion;
+        if $protocol == ProtocolVersion::RESP2 {
+            return Err(RedisError::from((
+                crate::ErrorKind::InvalidClientConfig,
+                "RESP3 is required for this command",
+            )));
+        }
+    };
+
+    ($protocol: expr, $message: expr) => {
+        use crate::types::ProtocolVersion;
+        if $protocol == ProtocolVersion::RESP2 {
+            return Err(RedisError::from((
+                crate::ErrorKind::InvalidClientConfig,
+                $message,
+            )));
+        }
+    };
+}
+
+pub(crate) use check_resp3;
+
 #[cfg(test)]
 mod tests {
     use super::*;
